@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2013,2014,2015,2016,2017,2018
+;;  Copyright (C) 2013,2014,2015,2016,2017,2018,2019
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -40,15 +40,16 @@
   #:use-module (web server)
   #:use-module (srfi srfi-1)
   #:use-module (ice-9 format)
-  #:re-export (;; page module
-               params
+  #:re-export (;; env
+               http-status
+
+               ;; page module
                response-emit
                throw-auth-needed
                tpl->html
                redirect-to
                tpl->response
                reject-method
-               response-error-emit
                run-after-request!
                run-before-response!
                run-when-sigint!
@@ -60,6 +61,7 @@
                post
                put
                patch
+               page-options
                page-delete
                :sql-mapping
                :str
@@ -123,12 +125,14 @@
                rc-mtime rc-mtime!
                rc-cookie rc-cookie!
                rc-set-cookie rc-set-cookie!
+               rc-oht rc-oht!
                rc-conn rc-conn!
                new-route-context
                route-context?
                get-header
                get-from-qstr
                get-referer
+               params
 
                ;; csv
                make-csv-reader
@@ -150,14 +154,12 @@
                mfd-simple-dump
                make-mfd-dumper
                content-type-is-mfd?
-               parse-mfd-body
                mfd
                call-with-mfd-data
                find-mfd
                make-mfd
                mfd?
                is-mfds?
-               mfds-count
                mfd-dispos
                mfd-name
                mfd-filename
