@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2014,2015,2016,2017,2018
+;;  Copyright (C) 2014,2015,2016,2017,2018,2019
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -24,6 +24,10 @@
 (define-module (artanis oht)
   #:use-module (artanis utils)
   #:use-module (artanis config)
+  #:use-module (artanis env)
+  #:use-module (artanis ssql)
+  #:use-module (artanis fprm)
+  #:use-module (artanis crypto base64)
   #:use-module (artanis sql-mapping)
   #:use-module (artanis db)
   #:use-module (artanis cookie)
@@ -43,6 +47,7 @@
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-9)
   #:use-module (srfi srfi-26)
+  #:use-module (srfi srfi-11)
   #:use-module (web uri)
   #:use-module (web request)
   #:export (define-handler
@@ -51,6 +56,7 @@
              post
              put
              patch
+             page-options
              page-delete
 
              :sql-mapping
@@ -71,20 +77,28 @@
              :websocket
              :lpc))
 
+(define (http-options-add! method rule)
+  (let* ((digest (string->sha-1 rule))
+         (ml (hash-ref *http-options-table* digest '(OPTIONS HEAD))))
+    (when (not (memq method ml))
+      (hash-set! *http-options-table* digest (cons method ml)))))
+
 (define (define-handler method raw-rule opts-and-handler)
   (let* ((rule (string-trim-right raw-rule #\/))
          (keys (rule->keys rule))
          (path-regexp (compile-rule rule))
          (opts (oah->opts opts-and-handler))
          (handler (oah->handler opts-and-handler)))
+    (http-options-add! method rule)
     (hash-set! *handlers-table*
                (cons method path-regexp)
-               (make-handler-rc handler keys (new-oht opts #:rule rule #:keys keys)))))
+               (make-handler-context handler keys (new-oht opts #:rule rule #:keys keys)))))
 
 (define (get rule . opts-and-handler) (define-handler 'GET rule opts-and-handler))
 (define (post rule . opts-and-handler) (define-handler 'POST rule opts-and-handler))
 (define (put rule . opts-and-handler) (define-handler 'PUT rule opts-and-handler))
 (define (patch rule . opts-and-handler) (define-handler 'PATCH rule opts-and-handler))
+(define (page-options rule . opts-and-handler) (define-handler 'OPTIONS rule opts-and-handler))
 ;; NOTE: delete method is rarely used, and this name will override the list deletion in Scheme.
 ;;       So my vote is changing its name, feel free to let me know if it's improper.
 (define (page-delete rule . opts-and-handler) (define-handler 'DELETE rule opts-and-handler))
@@ -103,20 +117,29 @@
   (map (lambda (m) (irregex-match-substring m 1))
        (artanis-list-matches *path-keys-regexp* rule)))
 
-(define-syntax-rule (get-oht rc)
-  (handler-rc-oht (get-handler-rc (rc-rhk rc))))
-
 ;; NOTE:
-;; Returns `values' proc if there's no such keyword was specified.
-;; Because we need to support the hooks, so there'd be a proc to receive
-;; multi-args when there's no such short-cut.
+;; 1. The situation to throw exception:
+;;    a. The keyword is invalid.
+;;    b. The rule didn't initialize the oht keyword.
+;; 2. Otherwise, we return `values', because we need to support the
+;;    hooks, and there'd be a proc to receive multi-args when there's
+;;    no such short-cut.
 (define-syntax-rule (=> opt rc args ...)
-  (let* ((oht (get-oht rc))
+  (let* ((oht (rc-oht rc))
          (h (and oht (hash-ref oht opt))))
-    (if h (h rc args ...) values)))
+    (if h
+        (h rc args ...)
+        (if (eq? opt #:cookies)
+            ;; #:cookies is special that no need to initialize first,
+            ;; and it could be used to operate the existing cookies
+            ;; from the requests.
+            values
+            (throw 'artanis-err 500 'oht-ref
+                   "The opt ~a isn't initialized for ~a"
+                   opt (rc-path rc))))))
 
 (define-syntax-rule (auth-enabled? rc)
-  (hash-ref (get-oht rc) #:auth))
+  (hash-ref (rc-oht rc) #:auth))
 ;; --------------------------------------------------------------
 ;; oht handlers
 
@@ -179,7 +202,6 @@
 ;; NOTE: Support cookies customization, which let users define special cookies.
 ;;       Like crypto cookies.
 (define (cookies-maker mode rule keys)
-  (define *the-remove-expires* "Thu, 01-Jan-70 00:00:01 GMT")
   (define current-cset! (make-parameter cookie-set!))
   (define current-cref (make-parameter cookie-ref))
   (define current-setattr (make-parameter cookie-modify))
@@ -195,11 +217,6 @@
     (rc-set-cookie! rc (map cdr ckl)))
   (define (setattr ckl ck . kargs)
     (apply (current-setattr) (cget ckl ck) kargs))
-  (define (remove rc k)
-    (let ((cookies (rc-set-cookie rc)))
-      (rc-set-cookie!
-       rc
-       `(,@cookies ,((current-maker) #:npv '((key "")) #:expires *the-remove-expires*)))))
   (define-syntax-rule (init-cookies names)
     (fold (lambda (ck p)
             (cons (cons ck ((current-maker))) p))
@@ -211,7 +228,6 @@
       ((ref) (cut cref ckl <> <>))
       ((setattr) (cut setattr ckl <> <...>))
       ((update) (cut update ckl <>))
-      ((remove) remove)
       (else (throw 'artanis-err 500 cookies-maker
                    "Invalid operation!" op))))
   (match mode
@@ -260,8 +276,10 @@
       (`(spawn ,sid) (cut %spawn <> #:idname sid <...>))
       (`(spawn ,sid ,proc) (cut %spawn <> #:idname sid #:proc proc <...>))
       (else (throw 'artanis-err 500 session-maker "Invalid config mode: ~a" mode))))
+  (define (get-sid rc idname)
+    (any (lambda (c) (and=> (cookie-ref c idname) car)) (rc-cookie rc)))
   (define (check-it rc idname)
-    (let* ((sid (any (lambda (c) (and=> (cookie-ref c idname) car)) (rc-cookie rc)))
+    (let* ((sid (get-sid rc idname))
            (session (session-restore (or sid ""))))
       (case session
         ((expired)
@@ -279,6 +297,12 @@
       (`(check-and-spawn ,sid)
        (or (check-it rc sid) (apply spawn-handler rc args)))
       ('spawn (apply spawn-handler rc args))
+      ('drop
+       (session-destory! (get-sid rc "sid"))
+       (:cookies-remove! rc "sid"))
+      (`(drop ,sid)
+       (session-destory! (get-sid rc sid))
+       (:cookies-remove! rc sid))
       (else (throw 'artanis-err 500 session-maker "Invalid call cmd: ~a" cmd)))))
 
 ;; for #:from-post
@@ -367,10 +391,13 @@
      (this-rule-enabled-websocket! rule 'raw))
     (('proto (? symbol? proto))
      ;; TODO: call protocol initilizer, and establish websocket for it.
-     ;; NOTE: In Artanis, we accept only one protocol for each URL-remapping,
+     ;; NOTE: By default, we accept only one protocol for each URL-remapping,
      ;;       if you have several protocols to service, please use different
      ;;       URL-remapping.
      (this-rule-enabled-websocket! rule proto))
+    (('proto (? symbol? proto) 'inexclusive)
+     ;; NOTE: Allow many clients subscribe to one named-pipe.
+     (this-rule-enabled-inexclusive-websocket! rule proto))
     (('redirect (? string? ip/usk))
      ;; NOTE: We use IP rather than hostname, since it's usually redirected to
      ;;       a LAN address. Using hostname may cause DNS issues.
@@ -405,6 +432,7 @@
       (match cmd
         (`(set ,key ,val) (lpc-set! lpc key val))
         (((or 'ref 'get) key) (lpc-ref lpc key))
+        ('(impl) (lpc-impl lpc))
         (else (throw 'artanis-err 500 gen-lpc-handler "Invalid cmd `~a'!" cmd)))))
   (match mode
     (#t
@@ -422,7 +450,152 @@
        (gen-lpc-handler rc cmd)))
     (else (throw 'artanis-err 500 lpc-maker "Invalid pattern `~a'!" mode))))
 
-;; ---------------------------------------------------------------------------------
+;; for #:with-auth
+;; NOTE: There's no :with-auth since it'll be triggerd by Artanis
+;;       automatically.
+(define (with-auth-maker mode rule keys)
+  (define (auth-action rc thunk failed-handler failed-url)
+    (if ((rc-oht-ref rc #:session) rc 'check)
+        (if (thunk? thunk)
+            (thunk)
+            (throw 'artanis-err 500 with-auth-maker
+                   "PATH: ~a, ~a is not a thunk!" (rc-path rc) thunk))
+        (failed-handler failed-url)))
+  (lambda (rc failed-handler thunk)
+    (match mode
+      (#t
+       (auth-action rc thunk failed-handler "/login"))
+      ((? string? failed-url)
+       (auth-action rc thunk failed-handler failed-url))
+      ('status
+       (auth-action rc thunk failed-handler 'status))
+      ((? procedure? generator)
+       (auth-action rc thunk failed-handler generator))
+      (else (throw 'artanis-err 500 with-auth-maker
+                   "Invalid mode `~a'~" mode)))))
+
+(define (auth-maker val rule keys)
+  (define-syntax-rule (->passwd rc passwd-field salt-field sql)
+    (let ((ret (DB-get-top-row (DB-query (DB-open rc) sql))))
+      (values (assoc-ref ret passwd-field)
+              (assoc-ref ret salt-field))))
+  (define-syntax-rule (post-ref post-data key)
+    (let ((ret (assoc-ref post-data key)))
+      (if ret
+          (car ret)
+          "")))
+  (define (basic-checker rc p sql passwd-field salt-field)
+    (let-values (((stored-pw _) (->passwd rc passwd-field salt-field sql)))
+      (string=? p stored-pw)))
+  (define (init-post rc)
+    (and (rc-body rc)
+         (generate-kv-from-post-qstr (rc-body rc))))
+  (define (default-hmac pw salt)
+    ;; We still have sha384, sha512 as options, but I think sha256 is enough.
+    (string->sha-256 (string-append pw salt)))
+  (define* (gen-result rc mode sql post-data
+                       #:key (hmac default-hmac) (checker #f)
+                       (passwd-field "passwd") (salt-field "salt"))
+    (define (table-checker)
+      (let-values (((stored-pw salt) (->passwd rc passwd-field salt-field sql)))
+        (cond
+         ((or (not stored-pw) (not salt))
+          (DEBUG "Stored PW: ~a, Salt: ~a, SQL: ~a~%" stored-pw salt sql)
+          #f)
+         (else
+          (string=? (hmac (post-ref post-data passwd-field)
+                          salt)
+                    stored-pw)))))
+    (define (run-checker)
+      (and (if checker (checker) (table-checker))
+           (and=> (rc-oht-ref rc #:session)
+                  (lambda (h) (h rc 'spawn)))))
+    (case mode
+      ((table) (run-checker))
+      ((table-specified-fields) (run-checker))
+      ((tpl) (run-checker))
+      ((basic)
+       (match (get-header rc 'authorization)
+         ;; NOTE: In match `=' opetator, the order of evaluation is from left to right.
+         ;;       So base64-decode will run first.
+         (`(basic . ,(= base64-decode-as-string (= (cut string-split <> #\:) up)))
+          (let ((u (car up)) (p (cadr up)))
+            (if checker
+                (checker rc u p)
+                (basic-checker rc p (sql u) passwd-field salt-field))))
+         (else #f)))
+      (else (throw 'artanis-err 500 auth-maker
+                   "Fatal BUG! Invalid mode! Shouldn't be here!" mode))))
+  (match val
+    (`(table ,table ,username-field ,passwd-field)
+     (lambda (rc . kargs)
+       (let* ((post-data (init-post rc))
+              (username (post-ref post-data username-field))
+              (passwd (post-ref post-data passwd-field))
+              (sql (->sql select `(,passwd-field salt) from table
+                          (where (string->keyword username-field)
+                                 username))))
+         (gen-result rc 'table-specified-fields sql post-data
+                     #:passwd-field passwd-field))))
+    (`(table ,table ,username-field ,passwd-field ,salt-field ,hmac)
+     (lambda (rc . kargs)
+       (let* ((post-data (init-post rc))
+              (username (post-ref post-data username-field))
+              (passwd (post-ref post-data passwd-field))
+              (sql (->sql select (list passwd-field salt-field) from table
+                          (where (string->keyword username-field)
+                                 username))))
+         (gen-result rc 'table-specified-fields sql post-data
+                     #:hmac hmac #:passwd-field passwd-field
+                     #:salt-field salt-field))))
+    (`(table ,table ,username-field ,passwd-field ,hmac)
+     (lambda (rc . kargs)
+       (let* ((post-data (init-post rc))
+              (username (post-ref post-data username-field))
+              (passwd (post-ref post-data passwd-field))
+              (sql (->sql select `(,passwd-field salt) from table
+                          (where (string->keyword username-field)
+                                 username))))
+         (gen-result rc 'table-specified-fields sql post-data
+                     #:hmac hmac #:passwd-field passwd-field))))
+    (`(table ,table ,hmac)
+     (lambda (rc . kargs)
+       (let* ((post-data (init-post rc))
+              (username (car kargs))
+              (passwd (cadr kargs))
+              (sql (->sql select '(passwd salt) from table
+                          (where #:username username))))
+         (gen-result rc 'table sql post-data #:hmac hmac))))
+    (`(basic ,table ,username-field ,passwd-field)
+     (lambda (rc . kargs)
+       (let* ((post-data (init-post rc))
+              (username (post-ref post-data username-field))
+              (passwd (post-ref post-data passwd-field))
+              (sql (->sql select `(,passwd-field salt) from table
+                          (where (string->keyword username-field)
+                                 username))))
+         (gen-result rc 'basic sql post-data #:passwd-field passwd-field))))
+    (`(basic ,checker)
+     (lambda (rc)
+       (gen-result rc 'basic #f #f #:checker checker)))
+    ((? string? tpl)
+     (lambda _
+       (make-db-string-template tpl)))
+    (('post username passwd checker)
+     (lambda* (rc #:optional (mode #t))
+       (when (not (rc-oht-ref rc #:from-post))
+         (init-oht! (rc-oht rc) #:from-post mode rule keys))
+       (let ((post-handler (rc-oht-ref rc #:from-post)))
+         (when (not post-handler)
+           (throw 'artanis-err 500 auth-maker
+                  "BUG: #:from-post should already be initialized but it seems not!"))
+         (let-values (((user pw) (apply post-handler rc `(get-vals ,username ,passwd))))
+           (and (checker user pw)
+                (and=> (rc-oht-ref rc #:session)
+                       (lambda (h) (h rc 'spawn))))))))
+    (else (throw 'artanis-err 500 auth-maker "Wrong pattern ~a" val))))
+
+;; -------------------------------------------------------------------
 
 ;; NOTE: these short-cut-maker should never be exported!
 ;; ((handler arg rule keys) rc . args)
@@ -433,151 +606,163 @@
 ;;    say, ${:xxx}, it means "to get xxx from rule key".
 ;; 2. Arounded with ${..}, and there's "@" as its prefix, say, ${@xxx},
 ;;    it means "to get xxx from body" (POST way).
-(define *options-meta-handler-table*
-  `(;; a short way for sql-mapping from the bindings in rule
-    ;; e.g #:sql-mapping "select * from table where id=${:id}"
-    (#:sql-mapping . ,sql-mapping-maker)
+(oh-define!
+ `(;; a short way for sql-mapping from the bindings in rule
+   ;; e.g #:sql-mapping "select * from table where id=${:id}"
+   (#:sql-mapping . ,sql-mapping-maker)
 
-    ;; generate a string from the bindings in rule
-    ;; e.g (get "/get/:who" #:str "hello ${:who}" ...)
-    (#:str . ,str-maker)
+   ;; generate a string from the bindings in rule
+   ;; e.g (get "/get/:who" #:str "hello ${:who}" ...)
+   (#:str . ,str-maker)
 
-    ;; short-cut for authentication
-    ;; #:auth accepts these values:
-    ;; 1. SQL as string template.
-    ;; 2. (table-name username-field passwd-field crypto-proc)
-    ;; 3. (table-name crypto-proc), so passwd field will be "passwd" in default.
-    ;; e.g (get "/auth"
-    ;;      #:auth "select ${passwd} from blog where usrname=${myname}"
-    ;;      (lambda (rc)
-    ;;       (:auth rc #:usrname (params rc "username") #:passwd (params rc "passwd"))
-    ;;       ...))
-    ;; The #:usrname and #:passwd will be expaned automatically.
-    ;; or (get "/auth" #:auth `(blog "myuser" "mypasswd" ,string->md5)
-    ;;     (lambda (rc)
-    ;;      (if (:auth rc)
-    ;;          (redirect-to rc "/dashboard")
-    ;;          (redirect-to rc "/login?auth_failed=true"))))
-    ;; or (get "/auth" #:auth `(blog ,string->md5)
-    ;;     (lambda (rc)
-    ;;      (if (:auth rc)
-    ;;          (redirect-to rc "/dashboard")
-    ;;          (redirect-to rc "/login?auth_failed=true"))))
-    (#:auth . ,auth-maker)
+   ;; short-cut for authentication
+   ;; #:auth accepts these values:
+   ;; 1. SQL as string template.
+   ;; 2. (table-name username-field passwd-field crypto-proc)
+   ;; 3. (table-name crypto-proc), so passwd field will be "passwd" in default.
+   ;; e.g (get "/auth"
+   ;;      #:auth "select ${passwd} from blog where usrname=${myname}"
+   ;;      (lambda (rc)
+   ;;       (:auth rc #:usrname (params rc "username") #:passwd (params rc "passwd"))
+   ;;       ...))
+   ;; The #:usrname and #:passwd will be expaned automatically.
+   ;; or (get "/auth" #:auth `(blog "myuser" "mypasswd" ,string->md5)
+   ;;     (lambda (rc)
+   ;;      (if (:auth rc)
+   ;;          (redirect-to rc "/dashboard")
+   ;;          (redirect-to rc "/login?auth_failed=true"))))
+   ;; or (get "/auth" #:auth `(blog ,string->md5)
+   ;;     (lambda (rc)
+   ;;      (if (:auth rc)
+   ;;          (redirect-to rc "/dashboard")
+   ;;          (redirect-to rc "/login?auth_failed=true"))))
+   (#:auth . ,auth-maker)
 
-    ;; Auto connection
-    ;; request a connection from connection-pool, and close automatically.
-    ;; NOTE: if you use #:sql-mapping short-cut, there's already a connect picked
-    ;;       from pool, so #:sql-mapping implies #:conn is set to #t.
-    ;; TODO: add recycling step after rule handler returning.
-    ;; e.g (get "/show-all" #:conn #t
-    ;;      (lambda (rc)
-    ;;       (:conn rc "select * from articles")))
-    (#:conn . ,conn-maker)
+   ;; Auto connection
+   ;; request a connection from connection-pool, and close automatically.
+   ;; NOTE: if you use #:sql-mapping short-cut, there's already a connect picked
+   ;;       from pool, so #:sql-mapping implies #:conn is set to #t.
+   ;; TODO: add recycling step after rule handler returning.
+   ;; e.g (get "/show-all" #:conn #t
+   ;;      (lambda (rc)
+   ;;       (:conn rc "select * from articles")))
+   (#:conn . ,conn-maker)
 
-    ;; Raw simple sql query
-    ;; NOTE: Raw sql is a const string of valid SQL.
-    ;; E.g: (get "/sqltest"
-    ;;       #:raw-sql "select * from Persons where name='nala'"
-    ;;       (lambda (rc)
-    ;;        (:raw-sql rc 'all)))
-    ;; It's used for a quick query, but you can't modify/specify query on the fly.
-    ;; Usage:
-    ;;  :raw-sql procedure accepts three modes:
-    ;; 1. 'all for getting all results.
-    ;; 2. certain number n for getting top n results.
-    ;; 3. 'top for getting the top result.
-    (#:raw-sql . ,raw-sql-maker)
+   ;; Raw simple sql query
+   ;; NOTE: Raw sql is a const string of valid SQL.
+   ;; E.g: (get "/sqltest"
+   ;;       #:raw-sql "select * from Persons where name='nala'"
+   ;;       (lambda (rc)
+   ;;        (:raw-sql rc 'all)))
+   ;; It's used for a quick query, but you can't modify/specify query on the fly.
+   ;; Usage:
+   ;;  :raw-sql procedure accepts three modes:
+   ;; 1. 'all for getting all results.
+   ;; 2. certain number n for getting top n results.
+   ;; 3. 'top for getting the top result.
+   (#:raw-sql . ,raw-sql-maker)
 
-    ;; Web caching
-    ;; The default value is #f.
-    ;; This is useful for web caching.
-    ;; values:
-    ;; * #t: will store the md5 of this page content,
-    ;;       and compare it with Etag.
-    ;; * #f: won't store md5, and won't do any caching work.
-    ;;       Etag would be ignored. If the current rule is for dynamic page,
-    ;;       you have to set it to #f.
-    ;; * filename: You may use (cache-file rc) to return the static file,
-    ;;             it's the same function like emit-response-with-file.
-    ;;             The difference is to cache it in memory.
-    ;; * Caveats:
-    ;; 1. #t will generate the dynamic page and get md5 for it anyway, the
-    ;;    only advantage would be saving the bandwidth (return 304).
-    ;; 2. Whether to use #:cache is depending on you.
-    ;; NOTE: this option will be disabled automatically for some reasons:
-    ;; 1. when #:auth was used in the same rule.
-    ;; 2. when there's any HTTP header to stop the caching.
-    (#:cache . ,cache-maker)
+   ;; Web caching
+   ;; The default value is #f.
+   ;; This is useful for web caching.
+   ;; values:
+   ;; * #t: will store the md5 of this page content,
+   ;;       and compare it with Etag.
+   ;; * #f: won't store md5, and won't do any caching work.
+   ;;       Etag would be ignored. If the current rule is for dynamic page,
+   ;;       you have to set it to #f.
+   ;; * filename: You may use (cache-file rc) to return the static file,
+   ;;             it's the same function like emit-response-with-file.
+   ;;             The difference is to cache it in memory.
+   ;; * Caveats:
+   ;; 1. #t will generate the dynamic page and get md5 for it anyway, the
+   ;;    only advantage would be saving the bandwidth (return 304).
+   ;; 2. Whether to use #:cache is depending on you.
+   ;; NOTE: this option will be disabled automatically for some reasons:
+   ;; 1. when #:auth was used in the same rule.
+   ;; 2. when there's any HTTP header to stop the caching.
+   (#:cache . ,cache-maker)
 
-    ;; short-cut to set cookies
-    ;; NOTE: you don't have to use :cookies-update! to sync cookies in rc,
-    ;;       there's a hook for that before response.
-    ;; e.g (get "/" #:cookies '(ca cb)
-    ;;      (lambda (rc)
-    ;;       (:cookies-set! rc 'ca "sid" "1231231")
-    ;;       "ok"))
-    (#:cookies . ,cookies-maker)
+   ;; short-cut to set cookies
+   ;; NOTE: you don't have to use :cookies-update! to sync cookies in rc,
+   ;;       there's a hook for that before response.
+   ;; e.g (get "/" #:cookies '(ca cb)
+   ;;      (lambda (rc)
+   ;;       (:cookies-set! rc 'ca "sid" "1231231")
+   ;;       "ok"))
+   (#:cookies . ,cookies-maker)
 
-    ;; Convert to certain MIME type
-    ;; There're three types: json/csv/xml/sxml
-    ;; NOTE: Only used at the end of the handler (to replace response-emit)
-    ;; e.g (get "/json" #:mime 'json
-    ;;       (lambda (rc)
-    ;;         (let ((j (json (object ("name" "nala") ("age" "15")))))
-    ;;           (:mime j))))
-    ;; e.g (get "/csv" #:mime 'csv
-    ;;       (lambda (rc)
-    ;;         (:mime '((a 1) (b 2)))))
-    ;; ==> "a,1\nb,2\n"
-    ;; e.g (get "/xml" #:mime 'xml
-    ;;       (lambda (rc)
-    ;;         (:mime '((a 1) (b 2)))))
-    ;; ==> "<a>1</a><b>2</b>"
-    ;; e.g (get "/sxml" #:mime 'sxml
-    ;;       (lambda (rc)
-    ;;         (:mime '((a 1) (b 2)))))
-    ;; ==> "((a 1) (b 2))"
-    (#:mime . ,mime-maker)
+   ;; Convert to certain MIME type
+   ;; There're three types: json/csv/xml/sxml
+   ;; NOTE: Only used at the end of the handler (to replace response-emit)
+   ;; e.g (get "/json" #:mime 'json
+   ;;       (lambda (rc)
+   ;;         (let ((j (json (object ("name" "nala") ("age" "15")))))
+   ;;           (:mime j))))
+   ;; e.g (get "/csv" #:mime 'csv
+   ;;       (lambda (rc)
+   ;;         (:mime '((a 1) (b 2)))))
+   ;; ==> "a,1\nb,2\n"
+   ;; e.g (get "/xml" #:mime 'xml
+   ;;       (lambda (rc)
+   ;;         (:mime '((a 1) (b 2)))))
+   ;; ==> "<a>1</a><b>2</b>"
+   ;; e.g (get "/sxml" #:mime 'sxml
+   ;;       (lambda (rc)
+   ;;         (:mime '((a 1) (b 2)))))
+   ;; ==> "((a 1) (b 2))"
+   (#:mime . ,mime-maker)
 
-    ;; Spawn sessions
-    (#:session . ,session-maker)
+   ;; Spawn sessions
+   (#:session . ,session-maker)
 
-    ;; Get data from POST
-    (#:from-post . ,from-post-maker)
+   ;; Get data from POST
+   (#:from-post . ,from-post-maker)
 
-    ;; Establish websocket channel
-    ;; Each time developers set :websocket and specifed a protocol, say, `myproto',
-    ;; Artanis will check if the file app/protocols/myproto.scm exists, then load
-    ;; the protocol initialize function in `myproto' which will also add `myproto'
-    ;; into *proto-table*. Then ragnarok will take charge of it to call the correct
-    ;; handlers.
-    ;; There're 3 cases:
-    ;; 1. Regular protocol handling over websocket
-    ;;    The initializer will create an instance for the protocol.
-    ;; 2. Redirect to another address (IP or UNIX socket)
-    ;;    The initializer will create a binding pair in redirect table:
-    ;;    src port and des port.
-    ;;    In this case, there's no protocol handling in Artanis. The protocol handler
-    ;;    is in remote (could be a host machine, or a process in current OS).
-    ;; 3. Proxy mode:
-    ;;    Different from reguar proxy, the proxy in Artanis runs over websocket, and
-    ;;    there's no extra listenning port, only 80/443.
-    ;;    In this case, there're 2 situations:
-    ;;    a. transparent proxy
-    ;;       No protocol handling at all, just redirecting the data without cooking.
-    ;;    b. cooked proxy
-    ;;       There's a bound protocol instance for it.
-    (#:websocket . ,websocket-maker)
+   ;; Establish websocket channel
+   ;; Each time developers set :websocket and specifed a protocol, say, `myproto',
+   ;; Artanis will check if the file app/protocols/myproto.scm exists, then load
+   ;; the protocol initialize function in `myproto' which will also add `myproto'
+   ;; into *proto-table*. Then ragnarok will take charge of it to call the correct
+   ;; handlers.
+   ;; There're 3 cases:
+   ;; 1. Regular protocol handling over websocket
+   ;;    The initializer will create an instance for the protocol.
+   ;; 2. Redirect to another address (IP or UNIX socket)
+   ;;    The initializer will create a binding pair in redirect table:
+   ;;    src port and des port.
+   ;;    In this case, there's no protocol handling in Artanis. The protocol handler
+   ;;    is in remote (could be a host machine, or a process in current OS).
+   ;; 3. Proxy mode:
+   ;;    Different from reguar proxy, the proxy in Artanis runs over websocket, and
+   ;;    there's no extra listenning port, only 80/443.
+   ;;    In this case, there're 2 situations:
+   ;;    a. transparent proxy
+   ;;       No protocol handling at all, just redirecting the data without cooking.
+   ;;    b. cooked proxy
+   ;;       There's a bound protocol instance for it.
+   (#:websocket . ,websocket-maker)
 
-    ;; Apply an instance of Local Persistent Cache
-    ;; This is useful when you want to store some key-value stuffs.
-    ;; There're 2 cases:
-    ;; 1. #t
-    ;;    Initialize lpc with default configuration, the backend is Redis, write-able.
-    ;; 2. (backend . args)
-    ;;    Specify the avalailable backend, and the args as the configuare options.
-    (#:lpc . ,lpc-maker)))
+   ;; Apply an instance of Local Persistent Cache
+   ;; This is useful when you want to store some key-value stuffs.
+   ;; There're 2 cases:
+   ;; 1. #t
+   ;;    Initialize lpc with default configuration, the backend is Redis, write-able.
+   ;; 2. (backend . args)
+   ;;    Specify the avalailable backend, and the args as the configuare options.
+   (#:lpc . ,lpc-maker)
+
+   ;; Auto-check authenticated session
+   ;; There're 2 modes:
+   ;; 1. #t
+   ;;    If failed then redirect to default /login?login_failed=true
+   ;; 2. failed-url
+   ;;    Specify the redirecting URL if failed
+   ;; NOTE:
+   ;; 1. #:with-auth must be cooperated with #:session
+   ;;    If you use #:with-auth without #:session, then it throws
+   ;;    an exception.
+   (#:with-auth . ,with-auth-maker)))
 
 (define-macro (meta-handler-register what)
   `(define-syntax-rule (,(symbol-append ': what) rc args ...)
@@ -607,18 +792,20 @@
 (define-syntax-rule (:cookies-update! rc)
   ((:cookies rc 'update) rc))
 (run-before-response! (lambda (rc body) (:cookies-update! rc)))
-(define-syntax-rule (:cookies-remove! rc k)
-  ((:cookies rc 'remove) k))
+(define (:cookies-remove! rc k)
+  (rc-set-cookie!
+   rc
+   `(,@(rc-set-cookie rc) ,(new-cookie #:npv `((,k . #f)) #:expires #t))))
 
-(define-syntax-rule (oh-ref o)
-  (assq-ref *options-meta-handler-table* o))
+(define (init-oht! oht key . args)
+  (hash-set! oht key (apply (oh-ref key) args)))
 
 ;; return #f when there's no opt specified
 (define* (new-oht opts #:key (rule 'no-rules) (keys 'no-keys))
-  (match opts
-    ((() ()) #f)
-    (else
-     (let ((oht (make-hash-table)))
+  (let ((oht (make-hash-table)))
+    (match opts
+      ((() ()) #f)
+      (else
        (apply
         for-each
         (lambda (k v)
@@ -626,6 +813,10 @@
             (cond
              ((or h (eq? k #:handler))
               ;; #:handler is user customed handler
+              (when (eq? k #:with-auth)
+                ;; If #:session and #:with-auth are not initialized
+                ;; simultaneously, then init it with default option
+                (init-oht! oht #:session #t rule keys))
               (hash-set! oht k (h v rule keys)))
              (else #f))))
         opts)
