@@ -31,6 +31,7 @@
                                  bytevector-u64-ref
                                  bytevector-u32-ref
                                  bytevector-u16-ref
+                                 make-bytevector
                                  put-u8
                                  put-bytevector
                                  get-bytevector-all
@@ -116,8 +117,40 @@
   ;; TODO: finish it
   #t)
 
-(define (send-websocket-closing-frame port)
-  (let ((close-frame (new-websocket-frame/client 'close #t #vu8(0))))
+(define (->bv code reason)
+  (let* ((bv (string->bytevector reason "iso-8859-1"))
+         (payload (make-bytevector (+ 2 (bytevector-length bv)) 0)))
+    ;; code must be network-byte-order (big-endian)
+    (bytevector-u8-set! payload 0 (bit-extract code 8 16))
+    (bytevector-u8-set! payload 1 (bit-extract code 0 8))
+    (for-each
+     (lambda (i) (bytevector-u8-set! payload (+ 2 i) (bytevector-u8-ref bv i)))
+     (iota (bytevector-length bv)))
+    payload))
+
+(define *websocket-status-code*
+  `((1000 . ,(->bv 1000 "Normal Closure"))
+    (1001 . ,(->bv 1001 "Going Away"))
+    (1002 . ,(->bv 1002 "Protocol error"))
+    (1003 . ,(->bv 1003 "Unsupported Data"))
+    (1004 . ,(->bv 1004 "Reserved"))
+    (1005 . ,(->bv 1005 "No status received"))
+    (1006 . ,(->bv 1006 "Abnormal Closure"))
+    (1007 . ,(->bv 1007 "Invalid frame payload data"))
+    (1008 . ,(->bv 1008 "Policy Violation"))
+    (1009 . ,(->bv 1009 "Message Too Big"))
+    (1010 . ,(->bv 1010 "Mandatory Ext."))
+    (1011 . ,(->bv 1011 "Internet Server Error"))
+    (1015 . ,(->bv 1015 "TLS handshake"))))
+
+(define* (send-websocket-closing-frame port #:key (status #f))
+  (define (gen-body)
+    (cond
+     (status
+      (let ((reason (assoc-ref status *websocket-status-code*)))
+        (or reason #vu8(0))))
+     (else #vu8(0))))
+  (let ((close-frame (new-websocket-frame/client 'close #t (gen-body))))
     (write-websocket-frame/client port close-frame)))
 
 (define *opcode-list*
@@ -277,8 +310,11 @@
     (cond
      ((< len 16bit-size) (put-bytevector port (uint-list->bytevector (list len) 'big 2)))
      ((< len 64bit-size) (put-bytevector port (uint-list->bytevector (list len) 'big 8)))
-     (else (throw 'artanis-err 500 write-payload-size
+     (else (throw 'artanis-err 1009 write-payload-size
                   "The payload size `~a' exceeded 64bit!" len))))
+  (when (port-closed? port)
+    (throw 'artanis-err 1001 write-websocket-frame/client
+           "The client port `~a' was closed!" port))
   (let* ((final? (websocket-frame/client-final? frame))
          (type (websocket-frame/client-type frame))
          (len (websocket-frame/client-length frame))
@@ -315,15 +351,15 @@
 (::define (print-websocket-frame frame)
   (:anno: (websocket-frame) -> ANY)
   (call-with-output-string
-    (lambda (port)
-      (let* ((head (websocket-frame-head frame))
-             (opcode (%get-opcode head))
-             (payload (websocket-frame-payload frame))
-             (size (websocket-frame-payload-length frame))
-             (mask (websocket-frame-mask frame)))
-        (format port "<websocket-frame:~%")
-        (format port "~10thead: ~a~%" head)
-        (format port "~10tfinal?: ~a~%" (is-final-frame? head))
-        (format port "~10ttype: ~a~%" (list-ref *opcode-list* opcode))
-        (format port "~10tpayload-size: ~a~%" size)
-        (format port "~10tpayload: ~a>" payload)))))
+   (lambda (port)
+     (let* ((head (websocket-frame-head frame))
+            (opcode (%get-opcode head))
+            (payload (websocket-frame-payload frame))
+            (size (websocket-frame-payload-length frame))
+            (mask (websocket-frame-mask frame)))
+       (format port "<websocket-frame:~%")
+       (format port "~10thead: ~a~%" head)
+       (format port "~10tfinal?: ~a~%" (is-final-frame? head))
+       (format port "~10ttype: ~a~%" (list-ref *opcode-list* opcode))
+       (format port "~10tpayload-size: ~a~%" size)
+       (format port "~10tpayload: ~a>" payload)))))
